@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -31,9 +32,9 @@ USA.
   (select-buffer (get-imail-browser-buffer url)))
 
 (define (get-imail-browser-buffer url)
-  (or (list-search-positive (buffer-list)
-	(lambda (buffer)
-	  (eq? (selected-container-url #f buffer) url)))
+  (or (find (lambda (buffer)
+	      (eq? (selected-container-url #f buffer) url))
+	    (buffer-list))
       (let ((container (open-resource url))
 	    (buffer
 	     (new-buffer
@@ -196,10 +197,10 @@ USA.
    (find-browsers-for container)))
 
 (define (find-browsers-for container)
-  (list-transform-positive (buffer-list)
-    (lambda (buffer)
-      (or (eq? (selected-container #f buffer) container)
-	  (memq container (browser-expanded-containers buffer))))))
+  (filter (lambda (buffer)
+	    (or (eq? (selected-container #f buffer) container)
+		(memq container (browser-expanded-containers buffer))))
+	  (buffer-list)))
 
 (define (browser-expanded-containers buffer)
   (buffer-get buffer 'IMAIL-BROWSER-EXPANDED-CONTAINERS '()))
